package sun.java2d.opengl;

import sun.java2d.pipe.RenderBuffer;
import java.security.AccessController;
import sun.misc.ThreadGroupUtils;
import sun.java2d.pipe.RenderQueue;

public class OGLRenderQueue extends RenderQueue
{
    private static OGLRenderQueue theInstance;
    private final OGLRenderQueue.QueueFlusher flusher;
    
    private OGLRenderQueue() {
        this.flusher = (OGLRenderQueue.QueueFlusher)AccessController.doPrivileged(()/*  */ -> new OGLRenderQueue.QueueFlusher(ThreadGroupUtils.getRootThreadGroup()));
    }
    
    public static synchronized OGLRenderQueue getInstance() {
        if (OGLRenderQueue.theInstance == null) {
            OGLRenderQueue.theInstance = new OGLRenderQueue();
        }
        return OGLRenderQueue.theInstance;
    }
    
    public static void sync() {
        if (OGLRenderQueue.theInstance != null) {
            OGLRenderQueue.theInstance.lock();
            try {
                OGLRenderQueue.theInstance.ensureCapacity(4);
                OGLRenderQueue.theInstance.getBuffer().putInt(76);
                OGLRenderQueue.theInstance.flushNow();
            }
            finally {
                OGLRenderQueue.theInstance.unlock();
            }
        }
    }
    
    public static void disposeGraphicsConfig(long n) {
        OGLRenderQueue instance = getInstance();
        instance.lock();
        try {
            OGLContext.setScratchSurface(n);
            RenderBuffer buffer = instance.getBuffer();
            instance.ensureCapacityAndAlignment(12, 4);
            buffer.putInt(74);
            buffer.putLong(n);
            instance.flushNow();
        }
        finally {
            instance.unlock();
        }
    }
    
    public static boolean isQueueFlusherThread() {
        return Thread.currentThread() == getInstance().flusher;
    }
    
    @Override
    public void flushNow() {
        try {
            this.flusher.flushNow();
        }
        catch (Exception ex) {
            System.err.println("exception in flushNow:");
            ex.printStackTrace();
        }
    }
    
    @Override
    public void flushAndInvokeNow(Runnable runnable) {
        try {
            this.flusher.flushAndInvokeNow(runnable);
        }
        catch (Exception ex) {
            System.err.println("exception in flushAndInvokeNow:");
            ex.printStackTrace();
        }
    }
    
    private native void flushBuffer(long p0, int p1);
    
    private void flushBuffer() {
        int position = this.buf.position();
        if (position > 0) {
            this.flushBuffer(this.buf.getAddress(), position);
        }
        this.buf.clear();
        this.refSet.clear();
    }
    
    static void access$100(OGLRenderQueue oglRenderQueue) {
        oglRenderQueue.flushBuffer();
    }
    
    private class QueueFlusher extends Thread
    {
        private boolean needsFlush;
        private Runnable task;
        private Error error;
        
        public QueueFlusher(ThreadGroup group) {
            super(group, "Java2D Queue Flusher");
            this.setDaemon(true);
            this.setPriority(10);
            this.start();
        }
        
        public synchronized void flushNow() {
            this.needsFlush = true;
            this.notify();
            while (this.needsFlush) {
                try {
                    this.wait();
                }
                catch (InterruptedException ex) {}
            }
            if (this.error != null) {
                throw this.error;
            }
            return;
        }
        
        public synchronized void flushAndInvokeNow(Runnable task) {
            this.task = task;
            this.flushNow();
        }
        
        @Override
        public synchronized void run() {
            int tryLock = 0;
            while (true) {
                if (!this.needsFlush) {
                    try {
                        tryLock = 0;
                        this.wait(100L);
                        if (!this.needsFlush && (tryLock = OGLRenderQueue.this.tryLock()) != 0) {
                            if (OGLRenderQueue.this.buf.position() > 0) {
                                this.needsFlush = true;
                            }
                            else {
                                OGLRenderQueue.this.unlock();
                            }
                        }
                        else {
                            continue;
                        }
                    }
                    catch (InterruptedException ex2) {}
                }
                else {
                    try {
                        this.error = null;
                        OGLRenderQueue.access$100(OGLRenderQueue.this);
                        if (this.task != null) {
                            this.task.run();
                        }
                    }
                    catch (Error error) {
                        this.error = error;
                    }
                    catch (Exception ex) {
                        System.err.println("exception in QueueFlusher:");
                        ex.printStackTrace();
                    }
                    finally {
                        if (tryLock != 0) {
                            OGLRenderQueue.this.unlock();
                        }
                        this.task = null;
                        this.needsFlush = false;
                        this.notify();
                    }
                }
            }
        }
    }
}
